/*
 * $Id: header.js 364428 2012-05-24 21:43:55Z meric $
 */

(function($) {

/*
 * $Id: config.js 445527 2014-09-04 19:10:29Z meric $
 */

jQuery.support.cors = true; // required for jQuery on old versions of IE

if (typeof TMS === 'undefined') {
    TMS = {
        UI: {}
    };
}

jQuery.extend(TMS, {
       uud_cgi_path: '/projects/uud/',
    base_path: '//www.ncbi.nlm.nih.gov/projects/genome/trackmgr/',
    cgi_path: '0.4/',
    cgis: {
        disptracks: 'disptracks.cgi',
        blasttracks: 'blasttracks.cgi',
        registertracks: 'createtracks.cgi',
        registerdl: 'registerdl.cgi',
        registerBlastRID: 'registerRID.cgi',
        trackdatacount: 'trackdatacount.cgi',
        trackdata: 'trackdata.cgi',
        trackbyacc: 'trackbyacc.cgi',
        trackbyid: 'trackbyid.cgi',
        facetset: 'facets.cgi',
        tracksearch: 'tracksearch.cgi',
        deregistertrack: 'removetrack.cgi',
        updatetrackattrs: 'updatetrackattrs.cgi'

        //switchcontext: 'switchcontext.cgi',
        //search: 'tracksearch.cgi'
    },
    uud_cgis: {
        'dl-meta': 'dl_meta.cgi',
        'sv-dl': 'sv_dl.cgi',
        'file-upload': 'ul/file_upload.cgi'
    },
    ncfetch_url: '/projects/sviewer/ncfetch.cgi',
    trackmgr_version: '0.4.1',
    logErrors: true,

    GetCgiUrl: function(cgi) {
 
        return TMS.base_path + TMS.cgi_path + TMS.cgis[cgi] 
      },

    GetUUDCgiUrl: function(cgi) {
        return TMS.uud_cgi_path + TMS.uud_cgis[cgi];
    },

    ValidateConfig: function(config, required) {
        var retval = {};
        if (!(config == null || required == null)) {
            var errors = [];
            jQuery.each(required, function(i, elem) {
                if (!config.hasOwnProperty(elem)) {
                    errors.push(new TMS.Error("Required attribute not specified: " + elem));
                }
            });
            if (errors.length > 0) {
                retval.errors = errors;
            }
        }
        return retval;
    }
});

/*
 * $Id: enums.js 442034 2014-07-29 21:58:03Z meric $
 */


TMS.Enum = function(proto, name)
{
    var enumName = name;
    jQuery.extend(this, proto);
    // Add uppercase properties - they are deprecated
    for (var prop in proto) {
        if (proto.hasOwnProperty(prop)) {
            var caps = prop.toUpperCase();
            this[caps] = this[prop];
        }
    }
    this._getEnumName = function() { return enumName; };
};

TMS.Enum.prototype = {
    IsValid: function(value) {
        for (var prop in this) {
            if (this.hasOwnProperty(prop) && value == this[prop]) {
                return true;
            }
        }
        return false;
    },
    ValidatedValue: function(value, defaultValue) {
        var val = defaultValue;
        var ok = defaultValue !== undefined ||
                 this.Default !== undefined ||
                 this.DEFAULT !== undefined;
        if (this.IsValid(value)) {
            val = value;
            ok = true;
        }
        else {
            val = val || this.Default || this.DEFAULT;
        }
        if (!ok && window.console && this._getEnumName) {
            window.console.log('*** Invalid value "' + value + '" for enum ' +
                               this._getEnumName()
                              );
        }
        return val;
    }
};

TMS.Status = new TMS.Enum({
    Ok: 'ok',
    Warnings: 'warning',
    Loading: 'loading',
    Errors: 'error',
    Failed: 'failed'
}, 'TMS.Status');

// status codes - internal
TMS.StatusCode = {
    ok: 0,
    warning: 1,
    errors: 2,
    failed: 3
};

TMS.Severity = new TMS.Enum({
    Default: 'error',
    warning: 'warning',
    info: 'info',
    error: 'error'
}, 'TMS.Severity');

TMS.TrackListType = {
    tms: 'tms',
    named: 'named',
    blast: 'blast',
    search: 'search'
};

/*
TMS.TrackType = {
    tms: 'tms',
    user: 'user',
    graph: 'graph'
};
*/

TMS.Position = new TMS.Enum({
    Default: 'bottom',
    'top': 'top',
    bottom: 'bottom'
}, 'TMS.Position');

TMS.RelativePosition = new TMS.Enum({
    before: 'before',
    after: 'after'
}, 'TMS.RelativePosition');

// upload status codes - internal
TMS.UploadStatus = {
    waiting: 0,
    loading: 1,
    complete: 2,
    warnings: 3,
    errors: 4,
    failed: 5,
    canceled: 6
};

/*
 * $Id: error.js 426249 2014-02-06 22:03:40Z meric $
 */

/*
 *  TrackMgr errors
 */


var nullfn = function() { };

if (window.console) {
    TMS.Log = function() {
        if (arguments.length === 1) {
            window.console.log(arguments[0]);
        }
        else if (arguments.length > 1) {
            window.console.log(arguments);
        }
    };
}
else {
    TMS.Log = nullfn;
}
if (window.console && jQuery.isFunction(window.console.error)) {
    TMS.LogError = function() {
        if (arguments.length === 1) {
            window.console.error(arguments[0]);
        }
        else if (arguments.length === 0) {
            window.console.error("Unspecified error");
        }
        else {
            window.console.error(arguments);
        }
    };
}
else {
    TMS.LogError = function(msg) {
        TMS.Log('*** ' + msg);
    }
}


TMS.Error = function(msg, severity)
{
    this.m_Severity = TMS.Severity.ValidatedValue(severity)
    this.m_Description = msg;
    this.m_Prefix = '';
}

TMS.Error.prototype = {
    SetPrefix: function(prefix) {
        this.m_Prefix = prefix;
    },

    toString: function() {
        return this.m_Severity + ': ' + this.m_Prefix + this.m_Description;
    },

    Log: function() {
        switch (this.m_Severity) {
        case TMS.Severity.error:
            TMS.LogError(this.toString());
            break;
        default:
            TMS.Log(this.toString());
            break;
        }
    }
};

TMS.ReportErrors = function(prefix, errors)
{
    if (errors.length > 0) {
        jQuery.each(errors, function(i, elem) {
            elem.SetPrefix(prefix);
            elem.Log();
        });
    }
};

/*
 * $Id: trackmgr.js 445550 2014-09-04 21:08:54Z meric $
 */

var TMSCache = function()
{
    this.m_Cache = {};
};

TMSCache.prototype =
{
    Get: function(key) {
        return this.m_Cache[key];
    },

    Store: function(key, value) {
        this.m_Cache[key] = value;
    },

    Remove: function(key) {
        delete this.m_Cache[key];
    }
};

TMS.TrackMgr =
{
    SetTrack: function(type, key, t) {
        if (type == null || key == null || t == null) {
            return;
        }
        TMS.TrackMgr.GetTICache(type).Store(key, t);
    },

    GetTrack: function(type, key) {
        if (type == null || key == null) {
            return null;
        }
        return TMS.TrackMgr.GetTICache(type).Get(key);
    },

    GetTrackInfo: function(t, datasource) {
        if (t == null) {
            return null;
        }
        if (datasource == null) {
            datasource = 'tmssvc';
        }
        var newtrackinfo = new TMS.TrackInfo(t);
        newtrackinfo.AddDataSource(datasource);
        var track_id = TMS.TrackInfo.ExtractTMSId(t);
        var tracktype = 'TMS';
        if (track_id !== newtrackinfo.GetTMSId()) {
            TMS.LogError('Internal error: track id mismatch [' + track_id + '] vs [' + newtrackinfo.GetTMSId() + ']');
            return null;
        }
        var trackinfo = TMS.TrackMgr.GetTrack(tracktype, track_id);
        if (trackinfo != null) {
            trackinfo.MergeAttrFrom(newtrackinfo);
            return trackinfo;
        }
        TMS.TrackMgr.GetTICache(tracktype).Store(track_id, newtrackinfo);
        return newtrackinfo;
    },

    RetrieveFullAttributes: function(ids, config) {
        var tmssvc_tracks = [];
        var tmssearch_tracks = [];
        var tracks = jQuery.each(ids, function(i, track_id) {
            var t = TMS.TrackMgr.GetTrack('TMS', track_id);
            if (t == null) {
                return;
            }
            if (!t.HasDataSource('tmssvc')) {
                tmssvc_tracks.push(t);
            }
            if (!t.HasDataSource('tmssearch')) {
                tmssearch_tracks.push(t);
            }
        });
        var promises = [];
        if (tmssvc_tracks.length > 0) {
            promises.push(TMS.GetTracksById(tmssvc_tracks, config));
        }
        if (tmssearch_tracks.length > 0) {
            var tids = jQuery.map(tmssearch_tracks, function(tr, i) { return tr.GetTMSId() }).join(',');
            promises.push(TMS.Search(jQuery.extend({
                accs: tids,
                return_facets: false,
                count: tmssearch_tracks.length
            }, config)));
        }
        return jQuery.when.apply(jQuery, promises);
    },

    ClearTrackCache: function(track_id) {
        if (track_id != null) {
            TMS.TrackMgr.GetTICache('TMS').Remove(track_id);
        }
    },

    GetTICache: function(type) {
        return TMS.TrackMgr.s_TICache[type];
    },

    s_TICache: {
        TMS: new TMSCache()
    }
};

/*
 * $Id: registertracks.js 448980 2014-10-10 02:05:14Z meric $
 */

//
// TMS.RegisterTracks -- register user tracks with TMS
// TMS.RegisterSVDLTracks -- register user tracks using svdl.cgi track metadata
//

// RecombineTracks combines non-assembly tracks ("non-displayable_track" type set by svdl) with
// an arbitrary assembly track. This avoids the one additional track created for uploads with features
// on assembly and non-assembly sequences
var RecombineTracks = function(tracks)
{
    if (tracks.length < 2) {
        return tracks;
    }
    var non_assm_trs = [];
    var assm_trs = [];
    jQuery.each(tracks, function(i, tr) {
        if (tr.track_type === "non-displayable_track") {
            non_assm_trs.push(tr);
        }
        else {
            assm_trs.push(tr);
        }
    });
    if (assm_trs.length === 0) {
        return non_assm_trs;
    }
    var tr_seq_ids = assm_trs[0].seq_ids;
    jQuery.each(non_assm_trs, function(i, tr) {
        jQuery.merge(tr_seq_ids, tr.seq_ids);
    });
    return assm_trs;
};

var FetchPrimerBLASTJobId = function()
{
    var $elem = jQuery('#breadcrumb');
    if ($elem.length > 0) {
        var arr = $elem.html().match(/Job id=([^\s<]+)/);
        return (arr && arr.length > 1 ? arr[1] : null);
    }
    return null;
};

var DeregisterUserTrack = function(track_id)
{
    var promise = new jQuery.Deferred();
    var params = {
        t: track_id
    };
    jQuery.ajax({
        dataType: 'json',
        async: true,
        type: 'POST',
        url: TMS.GetCgiUrl('deregistertrack'),
        data: params,
        xhrFields: { withCredentials: true }
    })
    .done(function(data, textStatus, jqXHR) {
        var errors = [];
        if (data.hasOwnProperty('messages')) {
            var errors = jQuery.map(data.messages, function(mesg, i) {
                if (mesg.hasOwnProperty('level') && mesg.level === 'error') {
                    return mesg.message;
                }
            });
            if (errors.length > 0) {
                promise.rejectWith(promise, [jqXHR, 'error', errors.join('\n')]);
                return;
            }
        }
        if (data.status === 'success') {
            TMS.TrackMgr.ClearTrackCache(track_id);
            promise.resolve();
        }
        else {
            promise.rejectWith(promise, [jqXHR, 'error', 'unable to deregister track [' + track_id + ']']);
        }
    })
    .fail(function(jqXHR, textStatus, errorThrown) {
        promise.rejectWith(promise, [jqXHR, 'error', 'unable to deregister track [' + track_id + ']: ' + errorThrown]);
    });
    return promise.promise();
};

var UpdateTrackAttributes = function(track_id, attrs)
{
    var promise = new jQuery.Deferred();
    var params = {
        t: track_id,
        attrs: attrs
    };
    jQuery.ajax({
        dataType: 'json',
        async: true,
        type: 'POST',
        url: TMS.GetCgiUrl('updatetrackattrs'),
        data: params,
        xhrFields: { withCredentials: true }
    })
    .done(function(data, textStatus, jqXHR) {
        if (data.status === 'success') {
            promise.resolve();
        }
        else {
            promise.rejectWith(promise, [jqXHR, 'error', 'unable to update track attributes [' + track_id + ']']);
        }
    })
    .fail(function(jqXHR, textStatus, errorThrown) {
        promise.rejectWith(promise, [jqXHR, 'error', 'unable to update track attributes [' + track_id + ']: ' + errorThrown]);
    });
    return promise.promise();
};

var CreateEmptyDeferred = function(args)
{
    var retval = new jQuery.Deferred(function(d) {
        if (args != null && jQuery.isArray(args)) {
            d.rejectWith(d, args);
        }
        else {
            d.reject();
        }
    });
    return retval.promise();
};

jQuery.extend(TMS, {
    RegisterBlastRID: function(rid) {
        var mesg = 'Blast RID registration not currently supported';
        new TMS.Error(mesg, TMS.Severity.error).Log();
        return CreateEmptyDeferred([null, 'error', mesg]);
/*
        if (rid == null || rid.length === 0) {
            var mesg = 'Blast RID not specified';
            new TMS.Error(mesg, TMS.Severity.error).Log();
            return CreateEmptyDeferred([null, 'error', mesg]);
        }
        var promise = new jQuery.Deferred();
        jQuery.ajax({
            dataType: 'json',
            async: true,
            type: 'POST',
            url: TMS.GetCgiUrl('registerBlastRID'),
            data: { rid: rid },
            xhrFields: { withCredentials: true }
        })
        .done(function(data, textStatus, jqXHR) {
            if (data.status === 'success') {
                promise.resolve();
            }
            else {
                promise.rejectWith(promise, [jqXHR, 'error', 'unable to register BLAST RID [' + rid + ']']);
            }
        })
        .fail(function(jqXHR, textStatus, errorThrown) {
            promise.rejectWith(promise, [jqXHR, 'error', 'unable to register BLAST RID [' + rid + ']: ' + errorThrown]);
        });
        return promise.promise();
*/
    },

    RegisterSADBTrack: function(sadb_accession, name, assembly, properties) {
        if (sadb_accession == null || sadb_accession.length === 0) {
            var mesg = 'SADB accession not specified';
            new TMS.Error(mesg, TMS.Severity.error).Log();
            return CreateEmptyDeferred([null, 'error', mesg]);
        }
        if (name == null || name.length === 0) {
            name = sadb_accession + '[SADB]';
        }
        var tattrs = [];
        if (properties != null) {
            jQuery.each(properties, function(key, value) {
                tattrs.push({ key: key, value: value });
            });
        }
        var track = {
            identity: { ids: [] }, // blank identity, cgi will set
            'track-name': name,
            'track-descr': name,
            'assembly-acc': assembly,
            'data-items': [
                {
                    datakey: sadb_accession,
                    seqids: []
                }
            ],
            'access-perms': { 'access-level': 'public' }
        };
        if (tattrs.length > 0) {
            track.attrs = jQuery.map(tattrs, function(item, k) {
                return ((item.value != null && item.value.length > 0) ? item : null);
            });
        }
        return TMS.RegisterTracks([track]);
    },

    x_AddRegisterCBs: function(xhr, promise) {
        return xhr.done(function(data, textStatus, jqXHR) {
            var errors = [];
            if (data.hasOwnProperty('messages')) {
                var errors = jQuery.map(data.messages, function(mesg, i) {
                    if (mesg.hasOwnProperty('level') && mesg.level === 'error') {
                        return mesg.message;
                    }
                });
                if (errors.length > 0) {
                    promise.rejectWith(promise, [jqXHR, 'error', errors.join('\n')]);
                    return;
                }
            }
            var tracks = [];
            if (data.hasOwnProperty('tracks')) {
                jQuery.each(data.tracks, function(i, tr) {
                    if (tr.hasOwnProperty('display_tracks')) {
                        jQuery.each(tr.display_tracks, function(j, dtr) {
                            tracks.push(TMS.TrackMgr.GetTrackInfo(dtr, 'tmssvc'));
                        });
                    }
                });
            }
            if (tracks.length > 0) {
                promise.resolve({ tracks: tracks });
            }
            else {
                promise.rejectWith(promise, [jqXHR, 'error', 'no tracks registered']);
            }
        })
        .fail(function(jqXHR, textStatus, errorThrown) {
            promise.rejectWith(promise, [jqXHR, 'error', 'unable to register tracks: ' + errorThrown]);
        });
    },

    RegisterTracks: function(tracks) {
        var req = {
            'CreateTracksCGI-Request': {
                'tracks': tracks
            }
        };
        var promise = new jQuery.Deferred();
        var xhr = jQuery.ajax({
            dataType: 'jsonp',
            async: true,
            type: 'POST',
            url: TMS.GetCgiUrl('registertracks'),
            data: { req: JSON.stringify(req) },
            xhrFields: { withCredentials: true }
        })
        this.x_AddRegisterCBs(xhr, promise);
        return promise.promise();
    },

    RegisterSVDLTracks: function(tracks, default_track_name, properties) {
        var Err = function(mesg, severity) {
            new TMS.Error(mesg, severity).Log();
            return CreateEmptyDeferred([null, 'error', mesg]);
        };
        if (tracks == null) {
            return Err('Data upload failed', TMS.Severity.error);
        }
        if (!tracks.hasOwnProperty('tracks')) {
            return Err('Data discovery failed', TMS.Severity.error);
        }
        if (properties == null) {
            properties = {};
        }

        var req = {
            dlkey: tracks.icache_key
        };
        if (default_track_name != null) {
            req.tname = default_track_name;
        }
        if (document.URL.search(/primertool.cgi/) > 10) {
            var jobid = FetchPrimerBLASTJobId();
            if (jobid !== null) {
                req.pb_jobid = jobid;
            }
        }

        var promise = new jQuery.Deferred();
        var xhr = jQuery.ajax({
            dataType: 'json',
            async: true,
            type: 'POST',
            url: TMS.GetCgiUrl('registerdl'),
            data: req,
            xhrFields: { withCredentials: true }
        });
        this.x_AddRegisterCBs(xhr, promise);
        return promise.promise();
    }
});

/*
 * $Id: trackinfo.js 443004 2014-08-08 19:54:22Z meric $
 */

//
// TMS.TrackInfo -- represents the track instance fully-qualified with attributes
//
var ITrackInfo = function(name)
{
    this.x_SetName(name);
};

ITrackInfo.prototype =
{
    GetName: function() {
        return this.m_Name;
    },

    GetGroup: function() {
        return null;
    },

    GetOrdering: function() {
        return null;
    },

    CanDeregister: function() {
        return false;
    },

    Deregister: function() {
        // create and return rejected deferred
        var that = this;
        var retval = new jQuery.Deferred(function(d) {
            d.rejectWith(that, [ 'Cannot deregister this track' ]);
        });
        return retval.promise();
    },

    x_SetName: function(name) {
        this.m_Name = name;
    },

    x_SetOrdering: function(ordering) {
        this.m_Ordering = ordering;
    },

    ToString: function() {
        return 'name: ' + this.GetName();
    }
};


// config is an ASN.1-derived JSON object
TMS.TrackInfo = function(config)
{
    this.x_SetName(config.name);
    this.m_Track_id = TMS.TrackInfo.ExtractTMSId(config);
    var attr_count = 0;
    if (config.hasOwnProperty('attrs')) {
        var attrs = this.m_Attrs = {};
        jQuery.each(config.attrs, function(i, attr) {
            attrs[attr.key] = attr.value;
            ++attr_count;
        });
    }
    if (config.hasOwnProperty('seqs')) {
        var counts = this.m_SeqidCounts = {};
        jQuery.each(config.seqs, function(i, seqcount) {
            var seqid_counts = counts['GI:' + seqcount.seq_id.gi] = {};
            jQuery.each(seqcount.counts.counts, function(i, count) {
                seqid_counts[count.type] = count.count;
            });
            if (seqcount.hasOwnProperty('seq_id2')) {
                var key = 'GI:' + seqcount.seq_id2.gi;
                if (!counts.hasOwnProperty(key)) {
                    counts[key] = {};
                }
                var seqid2_counts = counts[key];
                jQuery.each(seqcount.counts.counts, function(i, count) {
                    seqid2_counts[count.type] = count.count;
                });
            }
        });
    }
    if (config.hasOwnProperty('rid')) {
        this.m_RID = config.rid;
    }
    this.m_Datasources = {};
};

TMS.TrackInfo.ExtractTMSId = function(track)
{
    try {
        return track.dtrack_id.tag.str;
    }
    catch (e) {
        return null;
    }
};

TMS.TrackInfo.prototype = new ITrackInfo();

TMS.TrackInfo.prototype.GetTMSId = function()
{
    return this.m_Track_id;
};

TMS.TrackInfo.prototype.GetAttr = function(attr)
{
    return (this.m_Attrs == null) ? null : this.m_Attrs[attr];
};

TMS.TrackInfo.prototype.GetCounts = function()
{
    return this.m_SeqidCounts;
};

TMS.TrackInfo.prototype.GetCount = function(seqid, type)
{
    return (this.m_SeqidCounts == null) ? null : this.m_SeqidCounts[seqid][type];
};

TMS.TrackInfo.prototype.GetTotalCount = function(seqid)
{
    return this.GetCount(seqid, 'total');
};

TMS.TrackInfo.prototype.GetGroup = function()
{
    return this.GetAttr('group');
};

TMS.TrackInfo.prototype.GetOrdering = function()
{
    return this.GetAttr('track_order_by');
};

TMS.TrackInfo.prototype.AddDataSource = function(datasource)
{
    this.m_Datasources[datasource] = null;
};

TMS.TrackInfo.prototype.HasDataSource = function(datasource)
{
    return this.m_Datasources.hasOwnProperty(datasource);
};

TMS.TrackInfo.prototype.ToString = function()
{
    return 'track id: ' + this.GetId() +
           ', name: ' + this.GetName() +
           ', group: ' + this.GetGroup() +
           (jQuery.isFunction(Object.keys)
                ?  ', attr count: ' + Object.keys(this.m_Attrs).length
                : ''
           )
};

TMS.TrackInfo.prototype.CanDeregister = function()
{
    return true;
};

TMS.TrackInfo.prototype.IsBLASTTrack = function()
{
    return (this.m_RID != null);
};

TMS.TrackInfo.prototype.GetRID = function()
{
    return this.m_RID;
};

TMS.TrackInfo.prototype.Deregister = function()
{
    return this.CanDeregister()
               ? DeregisterUserTrack(this.m_Track_id)
               : ITrackInfo.prototype.Deregister.call(this);
};

//
// Adds and/or updates track attributes
//
TMS.TrackInfo.prototype.SetAttributes = function(attrs)
{
    if (attrs != null) {
        UpdateTrackAttributes(this.m_Track_id, attrs);
    }
};

TMS.TrackInfo.prototype.MergeAttrFrom = function(tinfo)
{
    if (tinfo != null && tinfo.m_Attrs != null) {
        var that = this;
        if (that.m_Attrs == null) {
            that.m_Attrs = {};
        }
        jQuery.each(tinfo.m_Attrs, function(key, value) {
            if (!that.m_Attrs.hasOwnProperty(key)) {
                that.m_Attrs[key] = value;
            }
        });
        jQuery.extend(that.m_Datasources, tinfo.m_Datasources);
    }
};
/*
 * $Id: tracklist.js 448980 2014-10-10 02:05:14Z meric $
 */

//TrackList(tracks: TrackInfo[], name: string, description: string);
//
// TMS.TrackList -- an unnamed track list
//
var TrackList = function(tracks, clone)
{
    this.m_Tracks = [];
    this.x_SetTracks(tracks, clone); // array of TMS.TrackInfo
};

TrackList.FromDispTracksCgi = function(obj, datasource)
{
    var tlists = [];
    if (obj.hasOwnProperty('display_tracks')) {
        tlists.push(new TrackList(jQuery.map(obj.display_tracks, function(track, i) {
            return TMS.TrackMgr.GetTrackInfo(track, datasource);
        })));
    }
    if (obj.hasOwnProperty('blast_tracks')) {
        TMS.LogError('Internal error: blast_tracks not supported');
    }
    return tlists;
};

TrackList.prototype =
{
    GetTracks: function() {
        return this.m_Tracks;
    },

/*
    // SwitchGenomeContext() changes the genome context for this AvailabeList object. This is only applicable to TMS tracks, as these have
    // correspondences between tracks in different genomic contexts (assemblies, chromosomes etc).
    SwitchGenomeContext: function(genome_ctx) {
        if (genome_ctx == null || this.m_TrackLists.length === 0) {
            return;
        }
    },
*/

    x_SetTracks: function(tr, clone) {
        if (tr == null) {
            return;
        }
        if (!jQuery.isArray(tr)) {
            TMS.LogError('Internal error: expected an array of tracks');
        }
        else if (clone == null || clone) {
            this.m_Tracks = jQuery.extend([], tr);
        }
        else {
            this.m_Tracks = tr;
        }
    }
};

//
// TMS.NamedTrackList
//
TMS.NamedTrackList = function(tracks, name, description)
{
    this.x_SetTracks(tracks);
    this.x_SetName(name);
    this.x_SetDesc(description);
}

TMS.NamedTrackList.prototype = new TrackList();

TMS.NamedTrackList.prototype.x_SetName = function(name) {
    this.m_Name = name;
}

TMS.NamedTrackList.prototype.x_SetDesc = function(desc) {
    this.m_Description = desc;
}

TMS.NamedTrackList.prototype.GetName = function()
{
    return this.m_Name;
}

TMS.NamedTrackList.prototype.GetDescription = function()
{
    return this.m_Description;
}

 
//
// TMS.IdTrackList
//

TMS.IdTrackList = function(ids, config)
{
    this.m_Promise = this.x_RetrieveTracks(ids, (config != null && config.hasOwnProperty('client')) ? config.client : null);
    if (config != null) {
        if (config.hasOwnProperty('allAttributes') && config.allAttributes) {
            var that = this;
            this.m_Promise = this.m_Promise.pipe(function() {
                return TMS.TrackMgr.RetrieveFullAttributes(ids, config);
            });
            // add a final promise which resolves using the tracklist as its context,
            // as expected by callers
            this.m_Promise = this.m_Promise.pipe(function() {
                return jQuery.Deferred().resolveWith(that);
            });
        }
        if (config.hasOwnProperty('onReady') && jQuery.isFunction(config.onReady)) {
            this.m_Promise.done(config.onReady);
        }
    }
}

TMS.IdTrackList.prototype = new TrackList();

TMS.IdTrackList.prototype.x_RetrieveTracks = function(ids, client)
{
    var that = this;
    var processAJAX = function(d, t, x) {
        if (d != null && d.hasOwnProperty('tracks') && d.tracks.hasOwnProperty('display_tracks')) {
            var tracks = jQuery.map(d.tracks.display_tracks, function(track, i) {
                return TMS.TrackMgr.GetTrackInfo(track, 'tmssvc');
            });
            that.x_SetTracks(tracks, false);
        }
    };

    if (ids.length === 0) {
        return new jQuery.Deferred();
    }
    var params = {
        id: ids.join(",")
    };
    if (client != null) {
        params.client = client
    }
    var dtrackajax  = {
        dataType: 'json',
        async: true,
        type: 'POST',
        processData: true,
        url: TMS.GetCgiUrl('trackbyid'),
        data: params
    };
    var promise = new jQuery.Deferred();
    jQuery.ajax(dtrackajax)
    .done(function(data, textStatus, jqXHR) {
        processAJAX(data, textStatus, jqXHR);
        promise.resolveWith(that);
    })
    .fail(function(jqXHR, textStatus, errorThrown) {
        var err = 'error retrieving tracks by id [' + textStatus + ']: ' + errorThrown;
        TMS.LogError(err);
        promise.rejectWith(that, [jqXHR, '', err]);
    })
    return promise.promise();
};

TMS.GetTracksById = function(ids, config)
{
    var tl = new TMS.IdTrackList(ids, config);
    return tl.m_Promise;
}


//
// TMS.DataAccTrackList
//

TMS.DataAccTrackList = function(accs, config)
{
    this.m_Promise = this.x_RetrieveTracks(accs, (config != null && config.hasOwnProperty('client')) ? config.client : null);
    if (config != null) {
        if (config.hasOwnProperty('allAttributes') && config.allAttributes) {
            var that = this;
            this.m_Promise = this.m_Promise.pipe(function() {
                var ids = jQuery.map(this.GetTracks(), function(track, i) {
                    return track.GetTMSId();
                });
                return TMS.TrackMgr.RetrieveFullAttributes(ids, config);
            });
            // add a final promise which resolves using the tracklist as its context,
            // as expected by callers
            this.m_Promise = this.m_Promise.pipe(function() {
                return jQuery.Deferred().resolveWith(that);
            });
        }
        if (config.hasOwnProperty('onReady') && jQuery.isFunction(config.onReady)) {
            this.m_Promise.done(config.onReady);
        }
    }
}

TMS.DataAccTrackList.prototype = new TrackList();

TMS.DataAccTrackList.prototype.x_RetrieveTracks = function(accs, client)
{
    var that = this;
    var processAJAX = function(d, t, x) {
        if (d != null && d.hasOwnProperty('tracks') && d.tracks.hasOwnProperty('display_tracks')) {
            var tracks = jQuery.map(d.tracks.display_tracks, function(track, i) {
                return TMS.TrackMgr.GetTrackInfo(track, 'tmssvc');
            });
            that.x_SetTracks(tracks, false);
        }
    };

    var params = {
        acc: accs.join(",")
    };
    if (client != null) {
        params.client = client
    }
    var dtrackajax  = {
        dataType: 'json',
        async: true,
        type: 'POST',
        processData: true,
        url: TMS.GetCgiUrl('trackbyacc'),
        data: params
    };
    var promise = new jQuery.Deferred();
    jQuery.ajax(dtrackajax)
    .done(function(data, textStatus, jqXHR) {
        processAJAX(data, textStatus, jqXHR);
        promise.resolveWith(that);
    })
    .fail(function(jqXHR, textStatus, errorThrown) {
        var err = 'error retrieving tracks by data accession [' + textStatus + ']: ' + errorThrown;
        TMS.LogError(err);
        promise.rejectWith(that, [jqXHR, '', err]);
    })
    return promise.promise();
};

TMS.GetTracksByAcc = function(accs, config)
{
    var datl = new TMS.DataAccTrackList(accs, config);
    return datl.m_Promise;
}

//
// TMS.Search --- track search
//


var kDefaultSearchParams =
{
    query: null,
    client: null,
    resource: null,
    assm_accs: null,
    filters: null,
    //filters: { facet: [attr1, attr2], facet1: [attr] }
    offset: 0,
    count: 10,
    result_cb: null,
    return_facets: true,
    return_zero_counts: false
};

var ProcessSearch = function(resource, qryparams)
{
    var sresult = new TMS.SearchResult();
    var processAJAX = function(facet_set, d, t, x) {
        if (d != null) {
            if (d.hasOwnProperty('tracks')) {
                sresult.m_TrackList = new TrackList();
                jQuery.each(TrackList.FromDispTracksCgi(d.tracks, 'tmssearch'), function(i, tl) {
                    sresult.m_TrackList.x_SetTracks(tl.GetTracks());
                    // expecting only a single track list
                    return false;
                });
            }
            if (facet_set != null && d.hasOwnProperty('stats')) {
                sresult.x_SetRecordCount(d.stats.record_count);
                if (d.stats.hasOwnProperty('facets')) {
                    var facetnames = [];
                    jQuery.each(d.stats.facets, function(i, f) {
                        var fields_complete = (f.hasOwnProperty('flags') && f.flags.fields_complete);
                        var fields = jQuery.map(f.fields, function(fld, i) {
                            return fld.field.hasOwnProperty('not_set')
                                       ? new TMS.FacetField(fld.count)
                                       : new TMS.FacetField(fld.count, fld.field.name);
                        });
                        var facet = facet_set.GetFacet(f.name);
                        if (facet != null) {
                            facetnames.push(f.name);
                            facet.SetFields(fields, fields_complete);
                        }
                        else {
                            TMS.LogError('Internal error: unrecognized facet [' + f.name + ']');
                        }
                    });
                    jQuery.each(facet_set.GetFacetsExcept(facetnames), function(i, f) {
                        f.ClearFields();
                    });
                    sresult.x_SetFacetSet(facet_set);
                }
            }
        }
    };

    var use_facets = !qryparams.hasOwnProperty('xf');
    var fs_promise = use_facets ? TMS.GetFacetSet(resource) : jQuery.Deferred().resolve();

    var dtrackajax  = {
        dataType: 'json',
        async: true,
        type: 'POST',
        data: qryparams,
        processData: true,
        url: TMS.GetCgiUrl('tracksearch'),
        xhrFields: { withCredentials: true }
    };
    var promise = new jQuery.Deferred();
    jQuery.ajax(dtrackajax)
    .done(function(data, textStatus, jqXHR) {
        if (data.hasOwnProperty('error')) {
            TMS.LogError(data.error);
            promise.rejectWith(sresult, [jqXHR, '', data]);
        }
        else {
            fs_promise.done(function() {
                var facet_set = use_facets ? this.Clone() : null;
                processAJAX(facet_set, data, textStatus, jqXHR);
                promise.resolveWith(sresult);
            });
        }
    })
    .fail(function(jqXHR, textStatus, errorThrown) {
        var err = 'error refreshing track list [' + textStatus + ']: ' + errorThrown;
        TMS.LogError(err);
        promise.rejectWith(sresult, [jqXHR, '', err]);
    })
    return promise.promise();
};

TMS.Search = function(params)
{
    var p = jQuery.extend({}, kDefaultSearchParams, params);
    var qparams =
    {
        accs: p.accs,
        t: p.query,
        client: p.client || '',
        resource: p.resource || null,
        assm_accs: p.assm_accs,
        offset: p.offset,
        pageSize: p.count
    };
    if (p.filters != null) {
        jQuery.each(p.filters, function(facet, values) {
            if (values != null) {
                var vals;
                if (jQuery.isArray(values)) {
                    vals = jQuery.map(values, function(val, i) {
                        return (val == null) ? '@null@' : val;
                    }).join('|');
                }
                else {
                    vals = values;
                }
                qparams[facet] = vals;
            }
        });
    }
    if (!(p.return_facets == null || p.return_facets)) {
        qparams.xf = '';
    }
    else {
        if (p.hasOwnProperty('search_facets')) {
            jQuery.each(p.search_facets, function(facet, prefix) {
                if (prefix.hasOwnProperty('length') && prefix.length > 0) {
                    qparams[facet + '-s'] = prefix;
                }
            });
        }
        if (p.return_zero_counts != null && p.return_zero_counts) {
            qparams.iz = '';
        }
    }
    // remove null values
    var qps = {};
    jQuery.each(qparams, function(k, v) {
        if (v != null) {
            qps[k] = v;
        }
    });
    var promise = ProcessSearch(qparams.resource, qps);
    if (p.result_cb != null && jQuery.isFunction(p.result_cb)) {
        promise.done(p.result_cb);
    }
    return promise;
};

TMS.SearchResult = function(qryparams)
{
    this.m_QueryParams = qryparams;
};

TMS.SearchResult.prototype =
{
    GetTrackList: function() {
        return this.m_TrackList;
    },

    GetFacetSet: function() {
        return this.m_FacetSet;
    },

    GetRecordCount: function() {
        return this.m_RecordCount;
    },

    x_SetRecordCount: function(count) {
        this.m_RecordCount = count;
    },

    x_SetFacetSet: function(facet_set) {
        this.m_FacetSet = facet_set;
    }
};

TMS.FacetField = function(count, field)
{
    this.m_Count = count;
    this.m_Field = field || null;
};

TMS.FacetField.prototype =
{
    GetField: function() {
        return this.m_Field;
    },

    GetCount: function() {
        return this.m_Count;
    }
};

TMS.Facet = function(internal_name, display_name, ordering, is_default, fields)
{
    this.m_Internal_name = internal_name;
    this.m_Display_name = display_name;
    this.m_Ordering = ordering;
    var ValToBool = function(val)
    {
        var t = jQuery.type(is_default);
        switch (t) {
        case 'null':
        case 'undefined':
            return false;
        case 'string':
            switch (val.toLowerCase()) {
            case 'true':
            case 'yes':
            case '1':
                return true;
            default:
                return false;
            }
        case 'boolean':
            return val;
        case 'number':
            return (val === 0) ? false : true;
        default:
            return false;
        }
    };

    this.m_Is_default = ValToBool(is_default);
    this.m_Fields = [];
    this.m_FieldByName = {};
    this.SetFields(fields);
};

TMS.Facet.prototype =
{
    GetInternalName: function() {
        return this.m_Internal_name;
    },

    GetDisplayName: function() {
        return this.m_Display_name;
    },

    GetOrdering: function() {
        return this.m_Ordering;
    },

    IsDefault: function() {
        return this.m_Is_default;
    },

    IsComplete: function() {
        return this.m_Is_complete;
    },

    ClearFields: function() {
        this.m_Fields = [];
        this.m_FieldsByName = {};
        this.m_Is_complete = false;
    },

    SetFields: function(fields, complete) {
        if (fields != null && jQuery.isArray(fields)) {
            if (complete != null) {
                this.m_Is_complete = Boolean(complete);
            }
            this.m_Fields = fields;
            var fldbyname = this.m_FieldByName = {};
            jQuery.each(fields, function(i, fld) {
                fldbyname[fld.GetField()] = fld;
            });
        }
    },

    GetFields: function() {
        return this.m_Fields;
    },

    GetField: function(name) {
        return this.m_FieldByName[name];
    }
};

TMS.GetFacetSet = function(resource)
{
    if (TMS.FacetSet.s_resourcecache.hasOwnProperty(resource)) {
        var fs = TMS.FacetSet.s_resourcecache[resource];
        var retval = new jQuery.Deferred(function(d) {
            d.resolveWith(fs);
        });
        return retval.promise();
    }

    var fs = new TMS.FacetSet(resource);
    var dtrackajax  = {
        dataType: 'xml',
        async: true,
        type: 'POST',
        data: { app: resource },
        processData: true,
        url: TMS.GetCgiUrl('facetset'),
        xhrFields: { withCredentials: true }
    };
    var promise = new jQuery.Deferred();
    jQuery.ajax(dtrackajax)
    .done(function(data, textStatus, jqXHR) {
        var facets = jQuery('facet', data).map(function(i, elem) {
            var f = $(this);
            return new TMS.Facet(f.children('query_name').text(),
                                 f.children('display_name').text(),
                                 f.children('display_order').text(),
                                 f.children('is_default').text()
                                );
        });
        fs.x_AddFacets(facets.get());
        TMS.FacetSet.s_resourcecache[resource] = fs;
        promise.resolveWith(fs);
    })
    .fail(function(jqXHR, textStatus, errorThrown) {
        var err = 'error retrieving facet list for application ' + resource + ' [' + textStatus + ']: ' + errorThrown;
        TMS.LogError(err);
        promise.rejectWith(fs, [jqXHR, '', err]);
    })
    return promise.promise();
};

TMS.FacetSet = function(resource, facets)
{
    this.m_Resource = resource;
    this.m_Facets = jQuery.isArray(facets) ? facets : [];
    this.m_FacetsByName = {};
};

TMS.FacetSet.s_resourcecache = {};

TMS.FacetSet.prototype =
{
    GetResource: function() {
        return this.m_Resource;
    },

    GetFacets: function() {
        return this.m_Facets;
    },

    // return all facets other than those in string array
    GetFacetsExcept: function(names) {
        var facets = [];
        var exceptnames = {};
        jQuery.each(names, function(i, facet) {
            exceptnames[facet] = null;
        });
        jQuery.each(this.m_FacetsByName, function(name, facet) {
            if (!exceptnames.hasOwnProperty(name)) {
                facets.push(facet);
            }
        });
        return facets;
    },

    GetFacet: function(internal_name) {
        return this.m_FacetsByName[internal_name];
    },

    Clone: function() {
        return jQuery.extend(true, {}, this);
    },

    x_AddFacets: function(facets) {
        if (facets != null && jQuery.isArray(facets)) {
            this.m_Facets = facets;
            var facetsbyname = this.m_FacetsByName = {};
            jQuery.each(facets, function(i, facet) {
                facetsbyname[facet.GetInternalName()] = facet;
            });
        }
    }
};

/*
 * $Id: availablelist.js 448980 2014-10-10 02:05:14Z meric $
 */

TMS.AvailableList = function(config)
{
    var retval = TMS.ValidateConfig(config, [ 'client', 'resource', 'assm_acc' ]);
    if (retval != null && retval.hasOwnProperty('errors')) {
        TMS.ReportErrors('TMS.AvailableList: ', retval.errors);
        return;
    }

    this.m_TrackLists = []; // array of TMS.TrackList
    var that = this;

    this.m_Client = config.client;
    this.m_Resource = config.resource;
    var that = this;
    this.m_Assm_acc = config.assm_acc;
    var promise = this.x_Refresh();
    if (config.hasOwnProperty('onReady') && config.onReady != null && jQuery.isFunction(config.onReady)) {
        promise.done(config.onReady);
    }
    if (config.hasOwnProperty('promises') && config.promises != null && jQuery.isArray(config.promises)) {
        config.promises.push(promise);
    }
};

TMS.AvailableList.DefaultTrackListSelector = function(tracklist)
{
    // exclude named lists
    return !(tracklist instanceof TMS.NamedTrackList);
};

TMS.AvailableList.UserTrackListSelector = function(tracklist)
{
    // only named lists
    return (tracklist instanceof TMS.NamedTrackList || tracklist instanceof TMS.IdTrackList);
};

TMS.AvailableList.NamedTrackListSelector = function(tracklist)
{
    // only named lists
    return (tracklist instanceof TMS.NamedTrackList);
};

TMS.AvailableList.DefaultTrackSelector = function(track)
{
    return true;
};

TMS.AvailableList.DefaultDisplayTrackSelector = function(track)
{
    return (track.GetAttr('show') === 'true');
};

TMS.AvailableList.UserUploadTrackSelector = function(track)
{
    return (track.GetGroup() === 'Uploaded Data');
};

var UniqueCopier = function(idfunc)
{
    this.m_GetIdFunc = idfunc || function(e) { return e; };
    this.m_Ids = {};
    this.m_Arr = [];
};

UniqueCopier.prototype =
{
    AddElements: function(els) {
        var that = this;
        jQuery.each(els, function(i, el) {
            var id = that.m_GetIdFunc(el);
            if (id != null && !that.m_Ids.hasOwnProperty(id)) {
                that.m_Arr.push(el);
                that.m_Ids[id] = null;
            }
        });
    },

    GetElements: function() {
        return this.m_Arr;
    }
};

var TrackIdExtractor = function(tr) {
    return tr.GetTMSId();
};

TMS.AvailableList.prototype =
{
    // manipulate/view the AvailableList.
    AddTrackList: function(tl) {
        if (tl != null) {
            if (!(tl instanceof TMS.NamedTrackList || tl instanceof TMS.IdTrackList)) {
                TMS.LogError('expected named or id track list', tl);
            }
            this.x_AddTrackList(tl);
        }
    },

    RemoveTrackList: function(namedtl) {
    },

    GetTrackLists: function(track_selector) {
        var tl_sel = jQuery.isFunction(track_selector) ? track_selector : TMS.AvailableList.UserTrackListSelector;
        return this.x_GetTrackListsForSelector(tl_sel);
    },

    x_GetTrackListsForSelector: function(tl_selector) {
        var tlist = jQuery.map(this.m_TrackLists, function(tl, i) {
            return (tl_selector(tl) ? tl : null);
        });
        return tlist;
    },
 
    // TrackListSelector { exclude_named: bool }
    // TrackSelector { 
    // Retrieve a single, flattened TrackList based upon this collection of
    // TrackList objects. By default, named lists are excluded
    GetUnifiedTrackList: function(tracklist_selector, track_selector) {
        var pick = function(sel, dflt) {
            return jQuery.isFunction(sel) ? sel : dflt;
        };
        var tl_sel = pick(tracklist_selector, TMS.AvailableList.DefaultTrackListSelector);
        var t_sel = pick(track_selector, TMS.AvailableList.DefaultTrackSelector);
        var uc = new UniqueCopier(TrackIdExtractor);
        jQuery.each(this.m_TrackLists, function(i, tlist) {
            if (tl_sel(tlist)) {
                jQuery.each(tlist.GetTracks(), function(i, track) {
                    var trackid = track.GetTMSId();
                    if (t_sel(track)) {
                        uc.AddElements([track]);
                    }
                });
            }
        });
        return new TrackList(uc.GetElements(), false);
    },
 
    // SwitchGenomeContext() changes the genome context for all TrackList objects.
    SwitchGenomeContext: function(genome_ctx) {
        if (genome_ctx == null || this.m_TrackLists.length === 0) {
            return;
        }
/*
        var ds = this;
        var params = jQuery.extend({
                client: this.m_Client,
                resource: this.m_Resource
            },
            genome_ctx.GetCgiParams()
        );
        jQuery.ajax({
            type: 'POST',
            dataType: 'json',
            async: true,
            url: TMS.GetCgiDir() + TMS.cgis.switchcontext,
            data: params,
            processData: true,
            success: function(d, t, x) {
                ds.x_ProcessTrackReply(d);
            },
            error: function(xhr, textStatus, errorThrown) {
                TMS.LogError('error in DisplayState.x_GetTracks()');
                //TMS.reportError.call(this, data, IDEO.AnnotationStatus.ERRORS, "BlastResult");
            }
        });
*/
    },

    Refresh: function() {
        return this.x_Refresh();
    },

    x_Refresh: function() {
        var that = this;
        var processAJAX = function(d, t, x) {
            that.m_TrackLists.length = 0; // clear the track list
            if (d != null && d.hasOwnProperty('tracks')) {
                jQuery.each(TrackList.FromDispTracksCgi(d.tracks, 'tmssvc'), function(i, tl) {
                    that.x_AddTrackList(tl);
                });
            }
        };

        var params = {
            client: this.m_Client,
            resource: this.m_Resource,
            assm_acc: this.m_Assm_acc
        };
        var dtrackajax  = {
            dataType: 'json',
            async: true,
            type: 'POST',
            processData: true,
            url: TMS.GetCgiUrl('disptracks'),
            data: params,
            xhrFields: { withCredentials: true }
        };
        var promise = new jQuery.Deferred();
        jQuery.ajax(dtrackajax)
        .done(function(data, textStatus, jqXHR) {
            processAJAX(data, textStatus, jqXHR);
            promise.resolveWith(that);
        })
        .fail(function(jqXHR, textStatus, errorThrown) {
            var err = 'error refreshing track list [' + textStatus + ']: ' + errorThrown;
            TMS.LogError(err);
            promise.rejectWith(that, [jqXHR, '', err]);
        })
        return promise.promise();
    },

    x_AddTrackList: function(tl) {
        if (tl != null) {
            this.m_TrackLists.push(tl);
            TMS.Log('AvailableList: added track list', tl);
        }
    }
};

function uniq(arr)
{
    var uc = new UniqueCopier();
    uc.AddElements(arr);
    return uc.GetElements();
};

TMS.GetUserTrackList = function(assm_accs)
{
    if (assm_accs == null) {
        TMS.LogError('TMS.AvailableList: assembly accession not specified');
        return jQuery.Deferred().reject();
    }
    if (!jQuery.isArray(assm_accs)) {
        assm_accs = [ assm_accs ];
    }
    switch (assm_accs.length) {
    case 0:
        TMS.LogError('TMS.AvailableList: assembly accession not specified');
        return jQuery.Deferred().reject();
    case 1:
        break;
    default:
        assm_accs = uniq(assm_accs);
    }

    var promises = [];
    var alists = jQuery.map(assm_accs, function(assm, i) {
        var cfg = {
            client: '',
            resource: '',
            assm_acc: assm,
            promises: promises
        };
        return new TMS.AvailableList(cfg);
    });
    var CombineTrackLists = function(trlists) {
        var uc = new UniqueCopier(TrackIdExtractor);
        jQuery.each(trlists, function(i, tl) {
            uc.AddElements(tl.GetUnifiedTrackList().GetTracks());
        });
        return new TrackList(uc.GetElements(), false);
    };
    // Return a promise for following steps: create then combine available lists. The returned promise
    // uses the combined track list as its context
    return jQuery.when.apply(jQuery, promises).pipe(function() {
        var tlist = CombineTrackLists(alists);
        return jQuery.Deferred().resolveWith(tlist);
    });
};

/*
 * $Id: dtrackinfo.js 423085 2013-12-30 21:02:27Z meric $
 */

TMS.DisplayTrackInfo = function(trackinfo)
{
    this.m_TrackInfo = trackinfo;
};

TMS.DisplayTrackInfo.prototype =
{
    GetName: function() {
        return this.m_TrackInfo.GetName();
    },

    GetTrackInfo: function() {
        return this.m_TrackInfo;
    },

    GetDisplayOptions: function() {
        return null;
    },

    GetDataFilters: function() {
        return null;
    }
};

/*
 * $Id: displaylist.js 431446 2014-04-03 16:43:59Z meric $
 */

TMS.DisplayList = function(available_list)
{
    if (available_list != null) {
        var tlist = available_list.GetUnifiedTrackList(null, TMS.AvailableList.DefaultDisplayTrackSelector);
        this.m_Tracks = jQuery.map(tlist.GetTracks(), function(ti, i) {
            return new TMS.DisplayTrackInfo(ti);
        });
    }
    else {
        this.m_Tracks = [];
    }
};

TMS.DisplayList.prototype =
{
    ClearTracks: function() {
        this.m_Tracks.length = 0;
    },

    // AddTrack: function(dtrackinfo: TMS.DisplayTrackInfo, position: TMS.Position)
    AddTrack: function(dtrackinfo, position) {
        if (dtrackinfo == null) {
            return;
        }
        if (!dtrackinfo instanceof TMS.DisplayTrackInfo) {
            console.error('expected TMS.DisplayTrackInfo', dtrackinfo);
        }
        position = TMS.Position.ValidatedValue(position);
        if (position === TMS.Position.bottom) {
            this.m_Tracks.push(dtrackinfo);
        }
        else {
            this.m_Tracks.unshift(dtrackinfo);
        }
    },

    // AddTrackRelative: function(dtrackinfo: TMS.DisplayTrackInfo,
    //                            position: TMS.DisplayTrackInfo,
    //                            relative: TMS.RelativePosition
    //                           )
    AddTrackRelative: function(dtrackinfo, position, relative) {
        if (dtrackinfo == null || position == null || relative == null) {
            return;
        }
        relative = TMS.Position.ValidatedValue(relative);
        if (relative == null) {
            return;
        }
        if (!dtrackinfo instanceof TMS.DisplayTrackInfo) {
            console.error('expected TMS.DisplayTrackInfo', dtrackinfo);
        }
        if (!position instanceof TMS.DisplayTrackInfo) {
            console.error('expected TMS.DisplayTrackInfo', position);
        }
        var idx = jQuery.inArray(dtrackinfo);
        if (idx === -1) {
            console.error('track not found in DisplayList', position);
            return;
        }
        if (relative === TMS.RelativePosition.after) {
            ++idx;
        }
        this.m_Tracks.splice(idx, 0, dtrackinfo);
    },

    GetTracks: function(clone) {
        if (clone == null || clone) {
            return jQuery.extend([], this.m_Tracks);
        }
        return this.m_Tracks;
    },

    Clone: function() {
        var dl = new TMS.DisplayList();
        jQuery.extend(dl.m_Tracks, this.m_Tracks);
    },

    Assign: function(src_displaylist) {
        if (src_displaylist == null) {
            return;
        }
        if (!src_displaylist instanceof TMS.DisplayList) {
            console.error('expected TMS.DisplayList', src_displaylist);
        }
        this.m_Tracks = jQuery.extend([], dl.m_Tracks);
    },

    AssignTracks: function(disptracks) {
        if (disptracks == null) {
            return;
        }
        if (!jQuery.isArray(disptracks)) {
            console.error('expected array of TMS.DisplayTrackInfo', disptracks);
        }
        this.m_Tracks = jQuery.extend([], disptracks);
    }
};

/*
 * $Id: upload.js 445537 2014-09-04 19:40:39Z meric $
 */

TMS.UseFileUploadCgi = true;

var dragEnabled = (typeof(FormData) !== 'undefined');
//var idx = 0;

var HasGB = function()
{
    return (typeof GB !== 'undefined');
}

var GBDebug = function()
{
    return (HasGB() && GB.uploadDebug != null && GB.uploadDebug());
}


var ParseMetaErrors = function(str)
{
    // split by <br><br>
    // remove multiple whitespaces
    str = str.replace(/<br><br>/g, "\n").replace(/<br>/g, " ").replace(/problem:/ig, "").replace(/[ ]{2,}/g, " ")
             .replace(/^[ ]+/g, "").replace(/\n[ ]+/g, "\n").replace(/\n$/g, "")
             .replace(/[ ]*line:[ ]*/gi, ", line ");
    return str.split("\n");
};

var ParseData = function(text)
{
    var data = null;
    try {
        eval("data=" + text.replace(/\&quot;/g, "'").replace(/[\r\n]/g, " ").replace(/\\/g, ""));
    }
    catch(e) {
        console.error(e);
    }
    return data;
};


var PollTimer = function(std_interval)
{
    this.m_StdInterval = std_interval;
    this.m_Timestamp = Date.now();
};

PollTimer.prototype =
{
    m_ProgressPct: 0,
    m_PollInterval: 2000, // min poll interval, ms
    GetPollInterval: function() {
        var f = 0.75;
        var pollTime = Date.now() - this.m_Timestamp;
        var tm;
        try {
            if (this.m_ProgressPct > 0) {
                tm = (1 - this.m_ProgressPct) * pollTime / this.m_ProgressPct;
                tm = Math.max(this.m_PollInterval, Math.min(tm, this.m_StdInterval * f));
            }
            else {
                tm = Math.max(this.m_PollInterval, this.m_StdInterval * f);
            }
        }
        catch(e) {
            console.error(e);
            tm = this.m_PollInterval;
        }
        return Math.round(tm);
    },

    SetProgressPct: function(pct) {
        m_ProgressPct = pct;
    }
};

var FileUploader = function(config, delegate)
{
    this.assm_acc = config.assm_acc;
    this.file = config.file;
    this.form = config.form;
    this.data = config.data;
    this.sadb = config.sadb;
    this.file_key = config.file_key;
    this.dataURL = config.dataURL;
    this.dataRID = config.dataRID;
    this.delegate = delegate;
    this.status = TMS.UploadStatus.waiting;
    if (this.file) {
        this.size = this.file.size || this.file.fileSize;
    }
    this.filename = config.filename;
    this.track_name = config.track_name;
    this.name = this.x_GetName();
    this.m_Promise = new jQuery.Deferred();
//    FileUploader[idx++] = this;
}

FileUploader.prototype =
{
    getPromise: function() {
        return this.m_Promise.promise();
    },

    setProgressBar: function(bar) {
        this._bar = bar;
        this._bar.setTitle(this.getName());
        this._bar.setCancelCallback(this.cancel, this);
    },

    getProgressBar: function() {
        return this._bar;
    },

    getName: function() {
        return this.name;
    },

    x_GetName: function() {
        if (this.sadb) {
            return this.sadb;
        }
        if (this.file) {
            return this.file.name;
        }
        if (this.track_name) {
            return this.track_name;
        }
        if (this.form) {
            return GetLastPathComponent(this.form.file.value);
        }
        if (this.dataURL) {
            return GetLastPathComponent(this.dataURL);
        }
        if (this.dataRID) {
            return "BLAST RID " + this.dataRID;
        }
        if (this.data) {
            return this.data.substr(0, this.m_NameLength).replace(/^[ \t\n\r]+|[ \t\n\r]+$/g, "").replace(/[ \t\n\r]+/g,"_") + "...";
        }
        return null;
    },

    setStatus: function(status) {
        if (status > this.status) {
            this.status = status;
            this.x_Conclude();
        }
    },
    getUploadStatus: function() {
        switch (this.status) {
        case TMS.UploadStatus.complete:
        case TMS.UploadStatus.warnings:
        case TMS.UploadStatus.errors:
            return "complete";
        case TMS.UploadStatus.failed:
            return "failed";
        case TMS.UploadStatus.canceled:
            return "canceled";
        default:
            return "loading";
        }
    },

    registerProperties: function(props) {
        this._props = props;
    },

    // accept from poll
    x_AcceptFromPoll: function(data, textStatus, jqXHR) {
        var self = this;
        if (this.isCanceled()) {
            if (this.needsCancel && data.job_status !== 'failed' && data.job_status !== 'canceled' && data.job_id) {
                setTimeout(function() {
                    self.x_CancelJob(data.job_id);
                }, 50);
                delete this.needsCancel;
            }
            return;
        }

        delete this.jobId;
        if (data.key) {
            this.m_PollDuration = Date.now() - this.m_StartTimestamp;
            if (GBDebug()) {
                console.debug('- total poll time: ' + String(this.m_PollDuration / 1000));
            }
            this.x_Fetch(data.key);
        }
        else if (data.job_status === 'failed' || data.job_status === 'canceled') {
            if (data.error_message) {
                console.error(data.error_message);
            }
            this.setStatus(TMS.UploadStatus.failed);
            this.finish();
        }
        else if (data.job_id) {
            var p;
            this.jobId = data.job_id;
            var bar = this.getProgressBar();
            if (data.progress_message) {
                try {
                    var msg = data.progress_message.replace(/\&quot;/g, "'");
                    eval('var d=' + msg);
                    if (d.percentage != null) {
                        var p = parseInt(d.percentage) / 100;
                        this.m_PollTimer.SetProgressPct(p);
                        if (bar != null) {
                            bar.setProgress(Math.max(1, p) * this.m_UploadSlice);
                        }
                    }
                    if (bar != null) {
                        bar.setStatus(d.current_task || 'pending');
                    }
                }
                catch(e) {
                    console.error(e);
                }
            }

            var tm = self.getPollInterval();
            if (GBDebug()) {
                console.debug('+ polling in ' + String(tm / 1000) + ' sec');
            }
            setTimeout(function() { self.x_PollStatus(data.job_id); }, tm);
        }
    },

    x_AddSVDLCallbacks: function(deferred) {
        var self = this;
        return deferred.done(function(data, textStatus, jqXHR) {
            self.x_AcceptFromUpload(data, textStatus, jqXHR);
        }).fail(function(jqXHR, textStatus, errorThrown) {
            if (textStatus === 'parsererror') {
                var data = ParseData(jqXHR.responseText);
                if (data != null) {
                    self.x_AcceptFromUpload(data, textStatus, jqXHR);
                    return;
                }
            }
            self.reportError(errorThrown, textStatus, jqXHR);
            self.setStatus(TMS.UploadStatus.failed);
            self.finish();
        });
    },

    x_PollStatus: function(job_id) {
        try {
            var deferred = jQuery.ajax({
                type: 'POST',
                async: true,
                dataType: 'json',
                url: TMS.GetUUDCgiUrl('sv-dl'),
                data: { job_key: job_id }
            });
            this.x_AddSVDLCallbacks(deferred);
        }
        catch(e) {
            console.error(e);
            this.reportError(e);
        }
    },

    // accept from upload
    x_AcceptFromUpload: function(data, textStatus, jqXHR) {
        if (GBDebug()) {
            console.debug("- upload time: " + String(this.m_UploadDuration / 1000));
        }
        this.m_StartTimestamp = Date.now();
        if (data.success == null || data.success) {
            this.x_AcceptFromPoll(data, textStatus, jqXHR);
        }
        else if (!this.isCanceled()) {
            console.error(data.msg);
            this.reportError(data.msg);
        }
    },

    x_Fetch: function(key) {
        var date = Date.now();
        try {
            var self = this;
            jQuery.ajax({
                type: 'POST',
                async: true,
                dataType: 'json',
                url: TMS.GetUUDCgiUrl('dl-meta'),
                data: { key: key }
            })
            .done(function(data, textStatus, jqXHR) {
                self.m_FetchDuration = Date.now() - date;
                if (GBDebug()) {
                    console.log("- meta fetch time: " + String(self.m_FetchDuration / 1000) + " sec");
                }
                if (!self.isCanceled()) {
                    self.x_AddTracks(data);
                }
            })
            .fail(function(jqXHR, textStatus, errorThrown) {
                if (!self.isCanceled()) {
                    self.reportError(errorThrown, textStatus, jqXHR);
                }
            });
        }
        catch(e) {
            console.error(e);
            this.reportError(e);
        }
    },

    reportError: function(errorThrown, textStatus, jqXHR) {
        textStatus = textStatus || "Error";
        console.error(textStatus + ": " + errorThrown);
        this.setStatus(TMS.UploadStatus.failed);
        this.addError(errorThrown);
        if (HasGB()) {
            GB.pingError(errorThrown, 'FileUpload');
        }
        this.finish();
    },

    x_CheckErrors: function(data) {
        if (data.error_msg) {
            this.setStatus(TMS.UploadStatus.errors);
            var errors = ParseMetaErrors(data.error_msg);
            for (var k = 0; k < errors.length; k++) {
                this.addError(errors[k]);
                console.error(errors[k]);
            }
        }
    },

    x_SetTrackData: function(tracks) {
        this.m_Tracks = tracks;
        this.m_TrackList = new TrackList(tracks);
    },

    x_AddRegisterCBs: function(xhr) {
        var date = Date.now();
        var that = this;
        xhr.done(function(data) {
            that.m_TrackDuration = Date.now() - date;
            that.x_SetTrackData(data.tracks);
            that.setStatus(TMS.UploadStatus.complete);
            that.finish();
        })
        .fail(function(jqXHR, textStatus, errorThrown) {
            if (!that.isCanceled()) {
                that.reportError(errorThrown, textStatus, jqXHR);
                that.setStatus(TMS.UploadStatus.failed);
                that.finish();
            }
        });
    },

    x_AddTracks: function(data) {
        this.x_CheckErrors(data);
        if (data.tracks) {
            try {
                var bar = this.getProgressBar();
                if (bar != null) {
                    bar.setStatus("Registering tracks");
                }

                var xhr = TMS.RegisterSVDLTracks(data, this.getName(), this._props);
                this.x_AddRegisterCBs(xhr);
            }
            catch(e) {
                console.error(e);
                if (!this.isCanceled()) {
                    this.reportError(e, "TMS");
                }
            }
        }
        else if (!this.isCanceled()) {
            var msg = data.msg || "Track discovery failed";
            var h = data.msg ? "Meta" : "TMS";
            this.reportError(msg, h);
        }
    },

    addError: function(error) {
        this.errors = this.errors || [];
        this.errors.push(error);
    },

    finish: function() {
        var bar = this.getProgressBar();
        if (bar != null) {
            bar.setFinished(this.status, this.errors);
        }
        if (this.delegate && this.delegate.finish) {
            // inform delegate
            this.delegate.finish(this);
        }
        if (GBDebug()) {
            GB.ping({
                jsevent: 'statistics',
                action: 'FinishUpload',
                upload: (this.m_UploadDuration ? this.m_UploadDuration / 1000 : "n/a"),
                poll: (this.m_PollDuration ? this.m_PollDuration / 1000 : "n/a"),
                fetch_meta: (this.m_FetchDuration ? this.m_FetchDuration / 1000 : "n/a"),
                register_tracks: (this.m_TrackDuration ? this.m_TrackDuration / 1000 : "n/a")
            });
            console.debug("FileUpload statistics: " + this.getName() + "\n" +
                (this.m_UploadDuration ? " - uploading:       " + String(this.m_UploadDuration / 1000) + " sec\n" : "") +
                (this.m_PollDuration ? " - polling:         " + String(this.m_PollDuration / 1000) + " sec\n" : "") +
                (this.m_FetchDuration ? " - fetch meta:      " + String(this.m_FetchDuration / 1000) + " sec\n" : "") +
                (this.m_TrackDuration ? " - register tracks: " + String(this.m_TrackDuration / 1000) + " sec\n" : "")
            );
        }
        this.x_Conclude();
    },

    x_Conclude: function() {
        switch (this.status) {
        case TMS.UploadStatus.failed:
        case TMS.UploadStatus.canceled:
            this.m_Promise.rejectWith(this);
            break;
        case TMS.UploadStatus.complete:
        case TMS.UploadStatus.warnings:
        case TMS.UploadStatus.errors:
            var tl = this.x_GetTrackList();
            this.m_Promise.resolve(tl);
            if (HasGB() && GB.Notification != null) {
	        // object is the notifier object, may be whatever 
	        // will update update Your Data (reloads AvailableList)
	        GB.Notification.postNotification("DL.UpdateUserTracks", this);
            }

            break;
        default:
        }
    },

    cancel: function() {
        if (this.jobId) {
            this.x_CancelJob(this.jobId);
        }
        else {
            this.needsCancel = true;
        }
        this.setStatus(TMS.UploadStatus.canceled);
        this.finish();
    },

    x_CancelJob: function(job_id) {
        console.debug("+ cancelling job '" + job_id + "'");
        try {
            jQuery.ajax({
                type: 'POST',
                async: true,
                dataType: 'json',
                url: TMS.GetUUDCgiUrl('sv-dl'),
                data: {
                    job_key: job_id,
                    cancel: 'cancel'
                }
            }).done(function(data, textStatus, jqXHR) {
                console.debug('+ job canceled: ' + JSON.stringify(data));
            }).fail(function(jqXHR, textStatus, errorThrown) {
                console.debug("+ failed to cancel job '" + job_id + "': " + testStatus + ', ' + errorThrown);
            });
        }
        catch(e) {
            console.error(e);
        }
    },

    isCanceled: function() {
        return (this.status === TMS.UploadStatus.canceled);
    },

    remove: function() {
        var bar = this.getProgressBar();
        if (bar != null) {
            bar.remove();
        }
    },

    x_ClearForm: function() {
        if (this.form) {
            var wrap = this.form.parentNode;
            if (wrap && wrap.parentNode) {
                wrap.parentNode.removeChild(wrap);
            }
        }
        delete this.form;
    },

    getTracks: function() {
        return this.m_Tracks;
    },

    x_GetTrackList: function() {
        return this.m_TrackList;
    },

    fileParams: {
    //    ret_meta_data: true,
        discover_track: true,
        data_action: "uploading",
        input_form: "file"
    },

    urlParams: {
    //    ret_meta_data: true,
        discover_track: true,
        data_action: "uploading",
        input_form: "url"
    },

    blastParams: {
    //    ret_meta_data: true,
        discover_track: true,
        data_action: "uploading",
        input_form: "rid"
    },

    filekeyParams: {
    //    ret_meta_data: true,
        discover_track: true,
        data_action: "uploading",
        input_form: "file_key"
    },

    // evaluate poll interval
    getPollInterval: function() {
        return this.m_PollTimer.GetPollInterval();
    },

    m_UploadSlice: 0.2,
    m_NameLength: 30
}

FileUploader.prototype.x_Submit = function(formData, date, settings)
{
    var bar = this.getProgressBar();
    if (bar != null) {
        bar.setProgress();
        bar.setStatus("pending");
    }
    var self = this;
    try {
        var deferred;
        if (TMS.UseFileUploadCgi && this.file_key == null) {
            deferred = jQuery.ajax(jQuery.extend({
                type: "POST",
                async: true,
                dataType: "json",
                url: TMS.GetUUDCgiUrl('file-upload'),
                contentType: false,
                //contentType: "multipart/form-data",
                processData: false,
                data: formData
            }, settings))
            .always(function(data, textStatus, jqXHR) {
                self.m_UploadDuration = Date.now() - date; // upload time
                self.m_PollTimer = new PollTimer(self.m_UploadDuration);
            })
            .done(function(data, textStatus, jqXHR) {
                self.file_key = data.key;
                self.filename = self.name;
                self.upload();
            });
        }
        else {
            deferred = jQuery.ajax(jQuery.extend({
                type: "POST",
                async: true,
                dataType: "json",
                url: TMS.GetUUDCgiUrl('sv-dl'),
                contentType: false,
                //contentType: "multipart/form-data",
                processData: false,
                data: formData
            }, settings))
            .always(function(data, textStatus, jqXHR) {
                if (!self.hasOwnProperty('m_PollTimer')) {
                    self.m_UploadDuration = Date.now() - date; // upload time
                    self.m_PollTimer = new PollTimer(self.m_UploadDuration);
                }
            });
            this.x_AddSVDLCallbacks(deferred);
        }
        deferred.always(function(data, textStatus, jqXHR) {
            self.x_ClearForm();
        });
    }
    catch(e) {
        console.error(e);
        this.reportError(e);
        //self.x_ClearForm();
    }
};

FileUploader.prototype.upload = function()
{
    var date = Date.now();

    this.setStatus(TMS.UploadStatus.loading);
    if (dragEnabled) {
        var appendFormValue = function(form_data, key, val) {
            if (!(form_data == null || key == null || val == null)) {
                form_data.append(key, val);
            }
        };
        var appendFormData = function(form_data, keys, params) {
            jQuery.each(keys, function(k, v) {
                if (params.hasOwnProperty(k)) {
                    appendFormValue(form_data, k, params[k]);
                }
            });
        };

        var formData;
        if (this.file_key) {
            formData = new FormData();
            appendFormValue(formData, 'assm_acc', this.assm_acc);
            appendFormValue(formData, 'file_key', this.file_key);
            appendFormValue(formData, 'file_name', this.filename);
            appendFormData(formData, this.filekeyParams, this.filekeyParams);
        }
        else if (this.file) {
            formData = new FormData();
            appendFormValue(formData, 'assm_acc', this.assm_acc);
            appendFormValue(formData, 'file', this.file, this.file.name);
            appendFormData(formData, this.fileParams, this.fileParams);
        }
        else if (this.data) {
            formData = new FormData();
            appendFormValue(formData, 'assm_acc', this.assm_acc);
            appendFormValue(formData, 'file', this.data);
            appendFormData(formData, this.fileParams, this.fileParams);
        }
        else if (this.form) {
            this.form.assm_acc.value = this.assm_acc;
            formData = new FormData(this.form);
        }
        else if (this.sadb) {
            this.x_SubmitSADB(this.sadb);
            return;
        }
        else if (this.dataURL) {
            formData = new FormData();
            appendFormValue(formData, 'assm_acc', this.assm_acc);
            appendFormValue(formData, 'url', this.dataURL);
            appendFormData(formData, this.urlParams, this.urlParams);
        }
        else if (this.dataRID) {
            formData = new FormData();
            appendFormValue(formData, 'assm_acc', this.assm_acc);
            appendFormValue(formData, 'rid', this.dataRID);
            appendFormData(formData, this.blastParams, this.blastParams);
        }
        else {
            throw 'FileUploader has no file neither form';
        }
        var expr = /^(?:.*,)?svdl(?:,.*)?$/; // svdl debug
        if (formData && HasGB() && GB.config && expr.test(GB.config.debug)) {
            appendFormValue(formData, 'save_all_data', 'true');
        }
        this.x_Submit(formData, date);
    }
    else if (this.data) {
        //var contentType = null; // "multipart/form-data";
        var formData = {
            file: this.data,
            assm_acc: this.assm_acc,
            discover_track: "true",
            input_form: "file"
        };

        this.x_Submit(formData, date, { processData: true });
    }
    else if (this.dataURL) {
        var formData = {
            url: this.dataURL,
            assm_acc: this.assm_acc,
            track_name: this.name,
            discover_track: 'true',
            input_form: 'url'
        };
        this.x_Submit(formData, date, { processData: true });
    }
    else if (this.dataRID) {
        var formData = {
            rid: this.dataRID,
            assm_acc: this.assm_acc,
            //track_name: this.name,
            discover_track: "true",
            input_form: "rid"
        };
        this.x_Submit(formData, date, { processData: true });
    }
    else if (this.form) {
        var self = this;
        var iframe = this.delegate.getIFrame();

        var uploadDidFinish = function(e) {
            try {
                self.m_UploadDuration = Date.now() - date;
                jQuery(iframe).unbind("load", uploadDidFinish);

                var re = /^[ \t\n]*[\{\[]/;
                var txt, data;
                txt = iframe.contentDocument ? iframe.contentDocument.body.innerHTML
                                             : iframe.contentWindow.document.body.innerHTML;
                if (re.test(txt)) { // begins with '{' or '['
                    try {
                        txt = jQuery.trim(txt);
                        data = JSON.parse(txt);
                        self.x_AcceptFromUpload(data);
                        self.x_ClearForm();
                    }
                    catch(e) {
                        console.error(e);
                        self.reportError(e);
                        self.x_ClearForm();
                    }
                }
                else {
                    var error = self.delegate.parseError(txt);
                    console.error("FileUpload: upload failed - " + (error || "unknown error"));
                    self.reportError(error || "upload failed");
                    self.x_ClearForm();
                }
            }
            catch(e) {
                console.error(e);
            }
        };

        var bar = this.getProgressBar();
        if (bar != null) {
            bar.setProgress();
            bar.setStatus("pending");
        }

        jQuery(iframe).bind("load", uploadDidFinish);
        this.form.assm_acc.value = this.assm_acc;
        this.form.action = TMS.GetUUDCgiUrl('sv-dl');
        this.form.submit();
    }
    else if (this.dataURL) {
        var formData = {
            url: this.dataURL,
            assm_acc: this.assm_acc,
            discover_track: 'true',
            input_form: 'url'
        };
        this.x_Submit(formData, date, { processData: true });
    }
    else if (this.dataRID) {
        var formData = {
            rid: this.dataRID,
            assm_acc: this.assm_acc,
            discover_track: 'true',
            input_form: 'rid'
        };
        this.x_Submit(formData, date, { processData: true });
    }
    else if (this.sadb) {
        this.x_SubmitSADB(this.sadb);
    }
    else {
        console.error('FileUpload: no upload form in outdated browser');
    }
};

FileUploader.prototype.x_SubmitSADB = function(sadb)
{
    if (sadb == null) {
        return;
    }
    try {
        var bar = this.getProgressBar();
        if (bar != null) {
            bar.setProgress();
            bar.setStatus("Registering tracks");
        }

        var xhr = TMS.RegisterSADBTrack(sadb, null, this.assm_acc, this._props)
        this.x_AddRegisterCBs(xhr);
    }
    catch(e) {
        console.error(e);
        if (!this.isCanceled()) {
            this.reportError(e, "TMS");
        }
    }
}

var GetLastPathComponent = function(path)
{
    if (path) {
        var index = path.lastIndexOf("/");
        if (index < 0) {
            index = path.lastIndexOf("\\");
        }
        if (index >= 0) {
            path = path.substr(index + 1);
        }
        index = path.indexOf("?", 1);
        if (index >= 0) {
            path = path.substr(0, index);
        }
    }
    return path;
};

TMS.FileUploader = FileUploader;
TMS.FileWrapper = FileUploader;

TMS.CreateTrackForDataKey = function(nc_key, track_name, settings)
{
    var ul = new FileUploader(jQuery.extend({
        filename: 'nckey ' + nc_key,
        file_key: nc_key,
        track_name: track_name
    }, settings));
    ul.upload();
    return ul.getPromise();
};

/*
 * $Id: ui-avlist.js 405579 2013-07-05 13:31:24Z meric $
 */

TMS.UI.AvailableList = function(availablelist, elem)
{
    this.m_AvailableList = availablelist;
    this.m_Accordion = jQuery(elem);
    this.x_AddTracks();
    jQuery(elem).accordion({
        collapsible: false,
        fillSpace: false
    });
};

TMS.UI.AvailableList.prototype.x_AddTracks = function()
{
    var sections = {};
    var sections_arr = [];
    jQuery.each(this.m_AvailableList.GetUnifiedTrackList().GetTracks(), function(i, tr) {
        var s = tr.GetGroup();
        var sect = sections[s];
        if (sect == null) {
            sect = sections[s] = {
                name: s,
                order_by: jQuery.isFunction(tr.GetAttr) ? tr.GetAttr('group_order_by') : Number.MAX_VALUE,
                tracks: []
            };
            sections_arr.push(sect);
        }
        sect.tracks.push(tr);
    });
    sections_arr.sort(function(a, b) {
        return (a.order_by - b.order_by); // numeric comparison, nulls ok
    });

    var accordion = this.m_Accordion;
    jQuery.each(sections_arr, function(i, s) {
        jQuery('<h3>').append(s.name).appendTo(accordion);
        var $s = jQuery('<div>').attr('data-section', s.name)
                                .attr('data-order', s.order_by)
                                .addClass('tracks-by-type')
                                .appendTo(accordion);
        s.tracks.sort(function(a, b) {
            return (a.GetOrdering() - b.GetOrdering()); // numeric comparison, nulls ok
        });
        jQuery.each(s.tracks, function(i, t) {
            jQuery('<div>').append(t.GetName())
                           .addClass('track')
                           .appendTo($s);
        });
    });
};

/*
 * $Id: footer.js 364428 2012-05-24 21:43:55Z meric $
 */

})(jQuery);

